@@ -2,6 +2,7 @@ from .util import get_data_yaml_dict, get_dynamo_table, replace_decimals
 from .cdc_parser import CDCParser
 from boto3.dynamodb.conditions import Key, Attr
 from decimal import Decimal
+from datetime import datetime
 import json
 import logging
 
@@ -19,13 +20,11 @@ class Dynamo:
         else:
             Dynamo.connections = get_data_yaml_dict(cls.__name__.lower(), environment)
 
-
     def get_cdc_data(self, customer_id):
         try:
             c = Dynamo.connections[cdc_connection]
-            dynamo_table = get_dynamo_table(c['access_key'], c['secret_access_key'], c['region'], c['cdc_dynamo_table'])
-            items = dynamo_table.scan(FilterExpression=Attr(c['cdc_customer_id_attr']).eq(customer_id))['Items']
-            return replace_decimals(items[0]) if len(items) > 0 else None
+            return self.get_item_by_last_date(cdc_connection, c['cdc_dynamo_table'], c['cdc_customer_id_attr'],
+                                              customer_id, c['cdc_date_attr'])
         except Exception as e:
             logging.exception("exception thrown in Dynamo.get_cdc_data")
             return None
@@ -84,6 +83,33 @@ class Dynamo:
             logging.exception("exception thrown in Dynamo.get_item")
             return None
 
+    def get_item_by_key_and_last_date(self, connection, table, pk_attr, pk_value, date_attr):
+        try:
+            c = Dynamo.connections[connection]
+            dynamo_table = get_dynamo_table(c['access_key'], c['secret_access_key'], c['region'], table)
+            items = replace_decimals(dynamo_table.query(KeyConditionExpression=Key(pk_attr).eq(pk_value))['Items'])
+            if len(items) > 0:
+                items = sorted(items, key=lambda x: datetime.strptime(x[date_attr], '%Y-%m-%dT%H:%M:%S.%fZ'))
+                return items[-1]
+            else:
+                return None
+        except Exception as e:
+            logging.exception("exception thrown in Dynamo.get_item_by_key_and_last_date")
+            return None
+
+
     def get_item_by_last_date(self, connection, table, attr, value, date_attr):
-        pass
+        try:
+            c = Dynamo.connections[connection]
+            dynamo_table = get_dynamo_table(c['access_key'], c['secret_access_key'], c['region'], table)
+            items = replace_decimals(dynamo_table.scan(FilterExpression=Attr(attr).eq(value))['Items'])
+            if len(items) > 0:
+                items = sorted(items, key=lambda x: datetime.strptime(x[date_attr], '%Y-%m-%dT%H:%M:%S.%fZ'))
+                return items[-1]
+            else:
+                return None
+        except Exception as e:
+            logging.exception("exception thrown in Dynamo.get_item_by_last_date")
+            return None
+
 
